@@ -7,7 +7,6 @@ import clease
 from clease_gui.base_dashboard import BaseDashboard
 from clease_gui.status_bar import update_statusbar
 from clease_gui import register_logger, utils
-import clease_gui as gui
 
 from . import fitting_widgets
 
@@ -48,8 +47,11 @@ class ECIFitDashboard(BaseDashboard):
         self.db_select_out = widgets.Output()
         self.db_select_widget = fitting_widgets.DBSelectWidget()
 
-        self.fit_eci_button = widgets.Button(description="Fit ECI", button_style="primary")
-        self.fit_eci_button.on_click(self._on_fit_eci_click)
+        self.fit_eci_button = utils.make_clickable_button(
+            self._on_fit_eci_click,
+            description="Fit ECI",
+            button_style="primary",
+        )
 
     def display(self):
         # self.update_algo_widgets()
@@ -140,7 +142,6 @@ class ECIFitDashboard(BaseDashboard):
         self.app_data[self.KEYS.EVALUATE] = evaluator
 
     @update_statusbar
-    @gui.utils.disable_cls_widget("fit_eci_button")
     def _on_fit_eci_click(self, b):
         with self.event_context(logger=logger) as cm:
             self.fit_eci()
