@@ -1,5 +1,6 @@
+import os
 import logging
-import time
+from time import perf_counter  # Import directly for faster access in main MC loop
 from datetime import timedelta
 import uuid
 from IPython.display import display, clear_output
@@ -11,9 +12,17 @@ from clease.montecarlo import constraints
 from clease_gui import register_logger, utils
 from clease_gui.base_dashboard import BaseDashboard
 from clease_gui.status_bar import update_statusbar
-from clease_gui.utils import make_clickable_button
+from clease_gui.timing import est_runtime_delta
 from clease.montecarlo import observers
 
+try:
+    from clease_cxx import has_parallel
+
+    HAS_PARALLEL = has_parallel()
+    del has_parallel  # Remove from namespace again.
+except ImportError:
+    HAS_PARALLEL = False
+
 __all__ = ["CanonicalMC"]
 
 logger = logging.getLogger(__name__)
@@ -77,16 +86,14 @@ class CanonicalMC(BaseDashboard):
 
         # MC logging frequency
         self.update_freq_widget = widgets.BoundedIntText(
-            value=30,
-            min=1,
+            value=500,
+            min=100,
             max=9999999,
-            description="Update freq. (sec):",
+            description="Update freq. (ms):",
             **self.DEFAULT_STYLE_KWARGS,
         )
 
-        self.update_freq_help_msg = widgets.Label(
-            value="Frequency of the logging from the MC run (in seconds)."
-        )
+        self.update_freq_help_msg = widgets.Label(value="Min frequency of MC status updates.")
 
         self.step_mode_help = widgets.Label(
             "1 sweep runs equals <number of atoms in supercell> steps."
@@ -132,16 +139,22 @@ class CanonicalMC(BaseDashboard):
 
         self.step_mode_widget.observe(_on_step_mode_change)
 
-        self.run_mc_button = make_clickable_button(
-            self._on_run_mc_click, description="Run MC", button_style="primary"
+        self.run_mc_button = self.make_event_button(
+            self._gui_run_mc,
+            description="Run MC",
+            button_style="primary",
+            # The calling function is in charge of locking this button
+            lock_button=False,
         )
 
         self.update_atoms_after_run_widget = widgets.Checkbox(
             description="Update supercell after MC run?", value=True
         )
 
-        self.clear_mc_app_data_button = widgets.Button(description="Clear MC data")
-        self.clear_mc_app_data_button.on_click(self._on_clear_mc_data)
+        self.clear_mc_app_data_button = self.make_event_button(
+            self._on_clear_mc_data,
+            description="Clear MC data",
+        )
 
         # Button for stopping the MC process
         self.stop_mc_button = widgets.Button(description="Abort MC", button_style="warning")
@@ -168,6 +181,15 @@ class CanonicalMC(BaseDashboard):
             **self.DEFAULT_STYLE_KWARGS,
         )
 
+        self.threads_widget = widgets.BoundedIntText(
+            min=1,
+            max=os.cpu_count(),
+            value=1,
+            description="# threads:",
+            disabled=not HAS_PARALLEL,
+            **self.DEFAULT_STYLE_KWARGS,
+        )
+
     def attach_constraints(
         self,
         mc: clease.montecarlo.Montecarlo,
@@ -210,12 +232,12 @@ class CanonicalMC(BaseDashboard):
 
         display(step_mode_box, self.steps_output)
         display(self.temperature_mode_widget, self.temp_mode_output)
-        display(self.step_observer_wdgt)
+        display(self.step_observer_wdgt, self.threads_widget)
         display(frq_box, self.constrain_sublattices_wdgt)
         display(self.clear_mc_app_data_button)
 
         display(run_mc_box)
-        mc_progress_box = widgets.HBox(
+        mc_progress_box = widgets.VBox(
             children=[
                 self.mc_progress,
                 self.mc_progress_label,
@@ -308,22 +330,24 @@ class CanonicalMC(BaseDashboard):
 
         # attach a CLEASE calculator
         atoms = clease.calculator.attach_calculator(settings, atoms, eci=eci)
+        if HAS_PARALLEL:
+            num_threads = self.threads_widget.value
+            logger.info("Setting number of threads to: %d", num_threads)
+            atoms.calc.set_num_threads(num_threads)
         mc = clease.montecarlo.Montecarlo(atoms, temp)
-        mc.status_every_sec = self.update_freq
 
         self.attach_constraints(mc, settings)
 
         return mc
 
-    def _on_clear_mc_data(self, b):
+    def _on_clear_mc_data(self):
         """Remove all data from previous MC runs"""
         try:
             del self.app_data[self.KEYS.CANONICAL_MC_DATA]
             logger.info("MC data has been cleared")
         except KeyError:
-            logger.info("No MC data to clear.")
             # There was no data
-            pass
+            logger.info("No MC data to clear.")
 
     def set_mc_data(self, data):
         self.app_data[self.KEYS.CANONICAL_MC_DATA] = data
@@ -364,7 +388,7 @@ class CanonicalMC(BaseDashboard):
             raise KeyError(f'Missing the following in app data: "{missing_keys}".')
 
     @update_statusbar
-    def _run_mc_in_thread(self) -> None:
+    def _gui_run_mc(self) -> None:
         """Start the `_run_mc` method in a separate thread,
         while also disabling the run mc button for the duration
         of the run
@@ -385,10 +409,6 @@ class CanonicalMC(BaseDashboard):
         temperatures = self.get_temp()
         n_temp = len(temperatures)
 
-        mc_app_data = self.get_mc_data()
-        data = {}
-        mc_app_data.append(data)
-
         # Initial atoms object, current working atoms
         atoms = self.atoms
 
@@ -420,19 +440,52 @@ class CanonicalMC(BaseDashboard):
         # Initialize the progress bar
         self.mc_progress.value = 0
         self.mc_progress.max = n_temp
-        start_time = time.perf_counter()
+        start_time = perf_counter()
         logger.info("Starting MC with %d temperature steps, and %d swaps per step.", n_temp, steps)
+
+        # Total number of swaps which will be performed.
+        tot_mc_steps = steps * n_temp
+
+        # Template for the progress bar label string
+        msg_template = f"""
+        {{prog:.1f}}%, {{ii}}/{n_temp}.
+        Current temp: {{temp:.1f}} K.
+        MC swaps: {{cur_mc_step:d}}/{tot_mc_steps:d}.
+        Runtime: {{dt}}. Remaining: {{estimated}}.
+        """
+        msg_template = msg_template.strip().replace("\n", " ")
+
+        def _update_progress_label(ii, temp, mc_step=None) -> None:
+            """Helper function to update the progress label."""
+            runtime = perf_counter() - start_time
+            dt = timedelta(seconds=int(runtime))
+            # Calculate the number of swaps that have been performed, so far.
+            cur_mc_step = ii * steps
+            if mc_step is not None:
+                cur_mc_step += mc_step.step
+            # prog = (ii + 1) / n_temp * 100  # Progress in percentage
+            prog = cur_mc_step / tot_mc_steps
+            # Estimated time remaining.
+            estimated = est_runtime_delta(prog, runtime)
+            # Fill in the remaining variables.
+            msg = msg_template.format(
+                prog=prog * 100,
+                ii=ii + 1,
+                temp=temp,
+                cur_mc_step=cur_mc_step,
+                dt=dt,
+                estimated=estimated,
+            )
+            self.mc_progress_label.value = msg
+
+        last_step_time = perf_counter()
+        update_freq = self.update_freq / 1000  # from ms to s
+        run_data = {}
         try:
             for ii, temp in enumerate(temperatures):
                 # Update progress bar
                 self.mc_progress.value += 1
-                prog = (ii + 1) / n_temp * 100  # Progress in percentage
-                runtime = time.perf_counter() - start_time
-                dt = timedelta(seconds=round(runtime))
-                self.mc_progress_label.value = (
-                    f"{prog:.1f}%, {ii+1}/{n_temp}. Runtime: {dt}. Current temp: {temp:.1f} K"
-                )
-
+                _update_progress_label(ii, temp)
                 # Initialize MC things
                 check_abort(mc)
                 mc.T = temp
@@ -450,50 +503,63 @@ class CanonicalMC(BaseDashboard):
                 # Simmulated annealing
                 logger.debug(
                     "Annealing: Temperature step %d of %d. Current temperature: %.3f K.",
-                    ii + 1,
+                    ii,
                     n_temp,
                     temp,
                 )
 
-                mc.run(steps)  # Execute the MC
+                # Execute the MC
+                for mc_step in mc.irun(steps):
+                    # Update the progress string every N seconds
+                    if perf_counter() - last_step_time > update_freq:
+                        _update_progress_label(ii, temp, mc_step=mc_step)
+                        last_step_time = perf_counter()
+                # One final update of the label.
+                _update_progress_label(ii, temp, mc_step=mc_step)
                 check_abort(mc)  # Check if we exited because of an abort
                 logger.debug("MC done after %d MC steps.", steps)
-                logger.debug("Energy before MC: %.3f eV.", initial_energy)
+                logger.debug("Energy before MC: %s eV.", initial_energy)
                 logger.debug(
                     "Lowest energy achieved during run: %.3f eV.",
                     emin_obs.lowest_energy,
                 )
-                self._update_mc_data(mc, emin_obs, data)
+                self._update_mc_data(mc, emin_obs, run_data)
 
             if self.update_after_run:
                 # Set the supercell to the best atoms object we found.
                 self.set_supercell(emin_obs.emin_atoms)
                 logger.info("Updated supercell to minimum energy configuration")
 
-            dt = timedelta(seconds=round(time.perf_counter() - start_time))
+            if run_data:
+                # Only append if we have any data.
+                mc_app_data = self.get_mc_data()
+                mc_app_data.append(run_data)
+
+            dt = timedelta(seconds=round(perf_counter() - start_time))
             dt_per_step = dt / n_temp
             logger.info("MC run complete in %s. Avg. time per temp: %s.", dt, dt_per_step)
         finally:
             self._mc = None
 
-    @update_statusbar
-    def _on_run_mc_click(self, b):
-        with self.event_context(logger=logger):
-            self._run_mc_in_thread()
-
     def _make_mc_data(self, mc: clease.montecarlo.Montecarlo, emin_obs) -> dict:
         """Create a dictionary with the relevant data from the MC run.
         We pass in the relevant emin observer as well, because we always want this data."""
         lowest_energy = emin_obs.lowest_energy
-        temp = mc.T
+        temp = mc.temperature
         accept_rate = mc.num_accepted / mc.current_step
-        mean_energy = mc.mean_energy.mean
+        thermo = mc.get_thermodynamic_quantities()
+
+        mean_energy = thermo["energy"]
+        heat_capacity = thermo["heat_capacity"]
+        en_var = thermo["energy_var"]
 
         data = dict(
             emin=lowest_energy,
             mean_energy=mean_energy,
             temperature=temp,
             accept_rate=accept_rate,
+            heat_capacity=heat_capacity,
+            en_var=en_var,
         )
         return data
 
