@@ -8,6 +8,9 @@ import ipywidgets as widgets
 import ase
 import ase.io
 import clease
+from clease.calculator import attach_calculator
+
+from clease_gui.app_data import Notification
 from clease_gui.base_dashboard import BaseDashboard
 from clease_gui.status_bar import update_statusbar
 from clease_gui.colors import bool2symbol
@@ -40,23 +43,37 @@ class SupercellDashboard(BaseDashboard):
 
         self.desc_label = widgets.Label(value=self.make_desc_label())
 
-        self.view_atoms_btn = utils.make_clickable_button(
+        self.view_atoms_btn = self.make_event_button(
             self._on_view_atoms_click,
             description="View Current Supercell",
             layout=self.DEFAULT_BUTTON_LAYOUT,
         )
 
-        self.load_supercell_button = widgets.Button(description="Load Supercell")
         self.load_supercell_text = widgets.Text(description="Filename:", value="supercell.traj")
-        self.load_supercell_button.on_click(self._on_load_supercell_click)
+        self.load_supercell_button = self.make_event_button(
+            self.load_supercell,
+            description="Load Supercell",
+        )
 
-        self.save_supercell_button = widgets.Button(description="Save Supercell")
         self.save_supercell_text = widgets.Text(description="Filename:", value="supercell.traj")
-        self.save_supercell_button.on_click(self._on_save_supercell_click)
+        self.save_supercell_button = self.make_event_button(
+            self.save_supercell, description="Save Supercell"
+        )
 
         self.species_output = widgets.Output()
         self.species_output.clear_output()
 
+        self.get_energy_btn = self.make_event_button(
+            self._on_get_energy_click,
+            description="Get Current Energy",
+            layout=self.DEFAULT_BUTTON_LAYOUT,
+        )
+        self._check_get_energy_btn()
+        self.app_data.subscribe_many(
+            [self.KEYS.SETTINGS, self.KEYS.SUPERCELL, self.KEYS.ECI],
+            self._supercell_dashboard_app_data_callback,
+        )
+
     def display(self):
         display(self.desc_label)
         vbox_sc_widgets = widgets.VBox(children=self.sc_widgets)
@@ -65,6 +82,7 @@ class SupercellDashboard(BaseDashboard):
             children=[
                 self.make_atoms_btn,
                 self.view_atoms_btn,
+                self.get_energy_btn,
             ]
         )
 
@@ -75,19 +93,47 @@ class SupercellDashboard(BaseDashboard):
         display(vbox_sc_widgets)
         display(self.species_output)
 
-    def sc_formula_unit(self):
-        try:
-            supercell = self.supercell
-        except ValueError:
-            return ""
-        else:
-            return supercell.get_chemical_formula()
+    def _check_get_energy_btn(self) -> None:
+        """Check whether the get_energy_btn is enabled or disabled."""
+        self.get_energy_btn.disabled = not (
+            self.app_data.get(self.KEYS.SETTINGS, False)
+            and self.app_data.get(self.KEYS.ECI, False)
+            and self.app_data.get(self.KEYS.SUPERCELL, False)
+        )
 
-    @update_statusbar
-    @utils.disable_cls_widget("load_supercell_button")
-    def _on_load_supercell_click(self, b):
-        with self.event_context(logger=logger):
-            self.load_supercell()
+    def _supercell_dashboard_app_data_callback(self, notification: Notification) -> None:
+        """Callback function to the AppData subscription."""
+        # Always check the get energy button
+        self._check_get_energy_btn()
+        if notification.key == self.KEYS.SUPERCELL:
+            new_val = notification.new_value
+            if new_val is not None:
+                self._update_after_supercell_change()
+
+    def _on_get_energy_click(self) -> None:
+        energy = self.get_energy_of_supercell()
+        supercell = self.app_data[self.KEYS.SUPERCELL]
+        energy_atom = energy / len(supercell)
+        logger.info(
+            "Current supercell energy: %.4f (eV). Energy per atom: %.4f eV/atom",
+            energy,
+            energy_atom,
+        )
+
+    def get_energy_of_supercell(self) -> float:
+        """Calculate the energy of the current supercell with the current settings and ECI"""
+        settings = self.app_data[self.KEYS.SETTINGS]
+        supercell = self.app_data[self.KEYS.SUPERCELL]
+        eci = self.app_data[self.KEYS.ECI]
+        atoms = attach_calculator(settings, supercell, eci=eci)
+        return atoms.get_potential_energy()
+
+    def sc_formula_unit(self) -> str:
+        """String of the formula unit of the supercell."""
+        supercell = self.app_data.get(self.KEYS.SUPERCELL, None)
+        if not supercell:
+            return ""
+        return supercell.get_chemical_formula()
 
     def load_supercell(self):
         fname = self.load_supercell_text.value
@@ -103,7 +149,7 @@ class SupercellDashboard(BaseDashboard):
         self.set_supercell(atoms)
         logger.info("Load successful.")
 
-    def make_random_supercell(self):
+    def make_random_supercell(self) -> None:
         atoms = self.settings.prim_cell.copy()
         atoms *= self.repetitions
 
@@ -113,19 +159,17 @@ class SupercellDashboard(BaseDashboard):
         supercell = self._make_atoms_at_conc(x)
         self.set_supercell(supercell)
 
-    @update_statusbar
-    @utils.disable_cls_widget("view_atoms_btn")
-    def _on_view_atoms_click(self, b):
-        with self.event_context(logger=logger):
-            self._visualize_supercell()
-            self._update_desc_label()
+    def _on_view_atoms_click(self):
+        self._visualize_supercell()
+        self._update_desc_label()
+        import time
+
+        time.sleep(5)
 
     def _visualize_supercell(self):
         from ase.visualize import view
 
         supercell = self.supercell
-        if supercell is None:
-            raise ValueError("No supercell has been made yet.")
         # use a copy to remove any energies, we don't want to see graphs
         # this is a bit of a hack...
         cpy = supercell.copy()
@@ -133,7 +177,8 @@ class SupercellDashboard(BaseDashboard):
         view(cpy)
         logger.info("Opened supercell in a new ASE GUI window.")
 
-    def make_desc_label(self):
+    def make_desc_label(self) -> str:
+        """Construct the string for the description label."""
         text = "Create a supercell."
         try:
             sc = self.supercell
@@ -148,17 +193,23 @@ class SupercellDashboard(BaseDashboard):
 
         return text
 
-    def _update_desc_label(self):
+    def _update_desc_label(self) -> None:
+        """Update the description label widget"""
         self.desc_label.value = self.make_desc_label()
 
-    def set_supercell(self, atoms: ase.Atoms, draw_species_output=True):
+    def _update_after_supercell_change(self, draw_species_output: bool = True) -> None:
+        """Make necessary adjustments to the dashboard after the supercell has changed."""
         # XXX: Do we want to set the active template here?
+        atoms = self.app_data[self.KEYS.SUPERCELL]
         self.settings.set_active_template(atoms)
-        self.app_data[self.KEYS.SUPERCELL] = atoms
         self._update_desc_label()
         if draw_species_output:
             self._make_species_output()
 
+    def set_supercell(self, atoms: ase.Atoms) -> None:
+        # The subscribtion to the AppData ensures _update_after_supercell_change will be called.
+        self.app_data[self.KEYS.SUPERCELL] = atoms
+
     @property
     def supercell(self):
         try:
@@ -203,12 +254,6 @@ class SupercellDashboard(BaseDashboard):
         ase.io.write(fname, supercell)
         logger.info("Save successful.")
 
-    @update_statusbar
-    @utils.disable_cls_widget("save_supercell_button")
-    def _on_save_supercell_click(self, b):
-        with self.event_context(logger=logger):
-            self.save_supercell()
-
     def get_species_per_basis(self) -> List[Dict[str, int]]:
         settings = self.settings
         supercell = self.supercell
@@ -253,11 +298,13 @@ class SupercellDashboard(BaseDashboard):
         if not self.settings.concentration.is_valid(index_by_basis, atoms):
             raise ValueError("The specified configuration violates the concentration constraints.")
         # We don't need to redraw the species output widgets again
-        self.set_supercell(atoms, draw_species_output=False)
+        # Adjust symbols in the supercell directly, avoid triggering subscriber.
+        self.app_data[self.KEYS.SUPERCELL].symbols = atoms.symbols
         logger.info("Supercell updated.")
 
     def _make_species_output(self):
         out = self.species_output
+        basis_elements = self.settings.basis_elements
 
         species_per_basis = self.get_species_per_basis()
 
@@ -269,7 +316,11 @@ class SupercellDashboard(BaseDashboard):
             for ii, species_count in enumerate(species_per_basis):
                 sublattice_index = ii + 1
                 # Draw the sublattice boxes, and add the callback so we can get number of species
-                output, callback = make_sublattice_box(species_count, sublattice_index)
+                output, callback = make_sublattice_box(
+                    species_count,
+                    sublattice_index,
+                    basis_elements[ii],
+                )
                 get_species_counts.append(callback)
                 display(output)
 
@@ -291,7 +342,9 @@ class SupercellDashboard(BaseDashboard):
 
 
 def make_sublattice_box(
-    species_count: Dict[str, int], sublattice_index: int
+    species_count: Dict[str, int],
+    sublattice_index: int,
+    basis_elements: List[str],
 ) -> Tuple[widgets.Output, Callable[[], Dict[str, int]]]:
     """Draw a box for the sublattice containing a given set of species.
     Returns the Output object with the boxes, as well as a function
@@ -348,7 +401,9 @@ def make_sublattice_box(
 
     # Create the individual species boxes
     with sublattice_out:
-        for species, count in species_count.items():
+        # One box per possible element
+        for species in basis_elements:
+            count = species_count.get(species, 0)
             wdgt, ratio_wdgt = _make_species_boxes(species, count, tot_in_basis)
             # We need to update tot_specified if we make a change here
             wdgt.observe(observe_change)
@@ -374,9 +429,9 @@ def make_sublattice_box(
         cur_count = sum(counts.values())
         if cur_count != tot_in_basis:
             raise ValueError(
-                (
-                    "Incorrect number of elements in sublattice {}." " Expected {}, but got {}."
-                ).format(sublattice_index, tot_in_basis, cur_count)
+                ("Incorrect number of elements in sublattice {}. Expected {}, but got {}.").format(
+                    sublattice_index, tot_in_basis, cur_count
+                )
             )
         return counts
 
